"""
PVT (pressure volume temperature) functions based on Black Oil PVT model for petroleum engineering calculations

Rinat Khabibullin
revision from 13/05/2022

Unifloc_VBA and unifloc_py refactoring
"""

import numpy as np
import neftpy.uconvert as uc
import scipy.optimize as opt

""" 
====================================================================================================
Корреляции расчета давления насыщения
====================================================================================================
"""

# векторизованная версия расчета 
def unf_pb_Standing_MPaa(rsb_m3m3=100, gamma_oil=0.86, gamma_gas=0.6, t_K=350):
    """
        bubble point pressure calculation according to Standing (1947) correlation

    :param rsb_m3m3: solution ration at bubble point, must be given, m3/m3
    :param gamma_oil: specific oil density (by water)
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :return: bubble point pressure abs in MPa

    ref1 "A Pressure-Volume-Temperature Correlation for Mixtures of California Oil and Gases",
    M.B. Standing, Drill. & Prod. Prac., API, 1947.

    ref2  "Стандарт компании Юкос. Физические свойства нефти. Методы расчета." Афанасьев В.Ю., Хасанов М.М. и др. 2002 г
    """

    min_rsb = 1.8
    rsb_old = np.copy(rsb_m3m3)
    rsb_m3m3 = np.where(rsb_m3m3 < min_rsb, min_rsb, rsb_m3m3)
    yg = 1.225 + 0.001648 * t_K - 1.769 / gamma_oil         # gas molar fraction
    pb_MPaa = 0.5197 * (rsb_m3m3 / gamma_gas) ** 0.83 * 10 ** yg
    # for low rsb values, we set the asymptotics Pb = 1 atma at Rsb = 0
    # for large rsb values do not correct what the correlation gives
    return np.where(rsb_old < min_rsb, 
                    (pb_MPaa - 0.1013) * rsb_old / min_rsb + 0.1013, 
                    pb_MPaa
                    )

# векторизованная версия
def unf_pb_Valko_MPaa(rsb_m3m3=100, gamma_oil=0.86, gamma_gas=0.6, t_K=350):
    """
        bubble point pressure calculation according to Valko McCain (2002) correlation

    :param rsb_m3m3: solution ration at bubble point, must be given, m3/m3
    :param gamma_oil: specific oil density (by water)
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :return: bubble point pressure abs in MPa

    ref SPE  "Reservoir oil bubblepoint pressures revisited; solution gas–oil ratios and surface gas specific gravities"
    W. D. McCain Jr.,P.P. Valko,
    """

    min_rsb = 1.8
    max_rsb = 800
    rsb_old = np.copy(rsb_m3m3)
    rsb_m3m3 = np.where(rsb_m3m3 < min_rsb, min_rsb, rsb_m3m3)
    rsb_m3m3 = np.where(rsb_m3m3 > max_rsb, max_rsb, rsb_m3m3)
    nplogrsb_m3m3 = np.log(rsb_m3m3)
    z1 = -4.81413889469569 + 0.748104504934282 * nplogrsb_m3m3 \
        + 0.174372295950536 * nplogrsb_m3m3 ** 2 - 0.0206 * nplogrsb_m3m3 ** 3
    z2 = 25.537681965 - 57.519938195 / gamma_oil + 46.327882495 / gamma_oil**2 \
         - 13.485786265 / gamma_oil ** 3
    z3 = 4.51 - 10.84 * gamma_gas + 8.39 * gamma_gas ** 2 - 2.34 * gamma_gas ** 3
    z4 = -7.2254661 + 0.043155 * t_K - 8.5548e-5 * t_K ** 2 + 6.00696e-8 * t_K ** 3
    z = z1 + z2 + z3 + z4

    pb_MPaa = 12.1582266504102 * np.exp(0.0075 * z**2 + 0.713 * z)

    """
    for low values of gas content we set the asymptotics Pb = 1 atm with Rsb = 0
    the Valko correlation is obtained using the GRACE nonparametric regression method (SPE 35412)
    The peculiarity of this approach is that beyond the adaptation interval the asymptotics are not observed
    therefore they are set manually
    for large values of gas content we continue the linear trend of correlation
    """

    pb_MPaa = np.where(rsb_old < min_rsb, 
                       (pb_MPaa - 0.1013) * rsb_old / min_rsb + 0.1013, 
                       np.where(rsb_old > max_rsb, 
                                (pb_MPaa - 0.1013) * rsb_old / max_rsb + 0.1013, 
                                pb_MPaa)
                        )

    return pb_MPaa

""" 
====================================================================================================
Корреляции расчета газосодержания (Gas Solution Ratio)
====================================================================================================
"""

# расчет c векторизацией
def unf_rs_Standing_m3m3(p_MPaa=1, pb_MPaa=10, rsb_m3m3=100, gamma_oil=0.86, gamma_gas=0.6, t_K=350):
    """
        Gas-oil ratio calculation inverse of Standing (1947) correlation for bubble point pressure

    :param p_MPaa: pressure, MPa
    :param pb_MPaa: buble point pressure, MPa
    :param rsb_m3m3: gas-oil ratio at the bubble point pressure, m3/m3
    :param gamma_oil: specific oil density (by water)
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :return: gas-oil ratio in m3/m3

    ref1 "A Pressure-Volume-Temperature Correlation for Mixtures of California Oil and Gases",
    M.B. Standing, Drill. & Prod. Prac., API, 1947.

    ref2  "Стандарт компании Юкос. Физические свойства нефти. Методы расчета." Афанасьев В.Ю., Хасанов М.М. и др. 2002 г
    может считать в случае если нет давления насыщения и газосодержания при давлении насыщения, корреляция не точная
    """
    
    yg = lambda t_K, gamma_oil: 1.225 + 0.001648 * t_K - 1.769 / gamma_oil
    rs_m3m3 = np.where(pb_MPaa * rsb_m3m3 == 0, 
                        gamma_gas * (1.92 * p_MPaa / 10 ** yg(t_K, gamma_oil)) ** 1.204, 
                        np.where(p_MPaa < pb_MPaa, 
                                rsb_m3m3 * np.divide(p_MPaa, pb_MPaa, 
                                                     where=pb_MPaa!=0
                                                     ) ** 1.204,
                                rsb_m3m3
                                )
                        )
    return rs_m3m3

def unf_rs_Velarde_m3m3(p_MPaa=1, pb_MPaa=10, rsb_m3m3=100., gamma_oil=0.86, gamma_gas=0.6, t_K=350):
    """
        Solution Gas-oil ratio calculation according to Velarde McCain (1999) correlation

    :param p_MPaa: pressure, MPa
    :param pb_MPaa: buble point pressure, MPa
    :param rsb_m3m3: gas-oil ratio at the bubble point pressure, m3/m3
    :param gamma_oil: specific oil density (by water)
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :return: gas-oil ratio in m3/m3

        ref1 "Correlation of Black Oil Properties at Pressures Below Bubblepoint Pressure—A New Approach",
    J. VELARDE, T.A. BLASINGAME Texas A&M University, W.D. MCCAIN, JR. S.A. Holditch & Associates, Inc 1999

    """

    pr = np.where(pb_MPaa > 0.101314, 
                  uc.MPa_2_psig(p_MPaa)/(uc.MPa_2_psig(pb_MPaa)), 
                  0
                  )

    _go_ = -0.929328621908127 + 1 / gamma_oil
    _t_ = 0.0039158526769204 * t_K  - 1
    _pb_ = pb_MPaa - 0.101352932209575

    a1 = (0.0849029848623362 * gamma_gas**1.672608 * _go_**0.92987 *_t_**0.247235 * _pb_**1.056052) 
    a2 = (1.20743882814017 * _go_ ** 0.337711 * _t_** 0.132795 * _pb_**0.302065) / (gamma_gas ** 1.00475)
    a3 = 0.231607087371213 * _pb_ ** 0.047094 / ( gamma_gas**1.48548 * _go_**0.164741 * _t_**0.09133)

    rs_m3m3 = np.where(pr <= 0, 
                       0.0,
                       np.where( pr < 1, 
                                (a1 * pr ** a2 + (1 - a1) * pr ** a3) * rsb_m3m3,
                                rsb_m3m3
                                ) 
                        )
    return rs_m3m3

def unf_rsb_Mccain_m3m3(rsp_m3m3=10, gamma_oil=0.86, psp_MPaa=0.0, tsp_K=0.0):
    """
        Solution Gas-oil ratio at bubble point pressure calculation according to McCain (2002) correlation
    taking into account the gas losses at separator and stock tank

    :param rsp_m3m3: separator producing gas-oil ratio, m3m3
    :param gamma_oil: specific oil density(by water)
    :param psp_MPaa: pressure in separator, MPaa
    :param tsp_K: temperature in separator, K
    :return: solution gas-oil ratio at bubble point pressure, rsb in m3/m3

    часто условия в сепараторе неизвестны, может считать и без них по приблизительной формуле

    ref1 "Reservoir oil bubblepoint pressures revisited; solution gas–oil ratios and surface gas specific gravities",
    J. VELARDE, W.D. MCCAIN, 2002
    """

    rsp_scfstb = uc.m3m3_2_scfstb(rsp_m3m3)
    api = uc.gamma_oil_2_api(gamma_oil)
    psp_psia = uc.MPa_2_psi(psp_MPaa)
    tsp_F = uc.K_2_F(tsp_K)

    nplogpsp_psia = np.log(psp_psia)
    nplogapi = np.log(api)
    z1 = -8.005 + 2.7 * nplogpsp_psia - 0.161 * nplogpsp_psia ** 2
    z2 = 1.224 - 0.5 * np.log(tsp_F)
    z3 = -1.587 + 0.0441 * nplogapi - 2.29e-5 * nplogapi ** 2
    z = z1 + z2 + z3
    rst_scfstb = np.exp(3.955 + 0.83 * z - 0.024 * z ** 2 + 0.075 * z ** 3)
    
    rsb = np.where(psp_MPaa * tsp_K > 0, 
                   rsp_scfstb + rst_scfstb,
                   np.where(rsp_m3m3 >= 0, 
                            1.1618 * rsp_scfstb,
                            0
                            )
                   )

    return uc.scfstb_2_m3m3(rsb)

""" 
====================================================================================================
Корреляции расчета объемного коэффициента нефти (Oil Formation Volume Factor FVF)
====================================================================================================
"""

def unf_bo_above_pb_m3m3(bob_m3m3=1.2, compr_o_1MPa=3e-3, pb_MPaa=10, p_MPaa=1):
    """
        Oil Formation Volume Factor according equation for pressure above bubble point pressure

    :param bob_m3m3: formation volume factor at bubble point pressure, m3m3
    :param compr_o_1MPa: weighted-average oil compressibility from bubblepoint pressure to a higher pressure of interest,1/MPa
    :param pb_MPaa: bubble point pressure, MPa
    :param p_MPaa: pressure, MPa
    :return: formation volume factor bo,m3m3

    ref1 book Mccain_w_d_spivey_j_p_lenn_c_p_petroleum_reservoir_fluid,third edition, 2011

    ! Actually, this correlation is belonged ro Vasquez & Beggs (1980). In some sources is
    noted that this is Standing correlation.

    ref2 Vazquez, M. and Beggs, H.D. 1980. Correlations for Fluid Physical Property Prediction.
    J Pet Technol 32 (6): 968-970. SPE-6719-PA

    """

    return np.where(p_MPaa <= pb_MPaa, bob_m3m3, bob_m3m3 * np.exp(compr_o_1MPa * (pb_MPaa - p_MPaa)))

def unf_bo_below_pb_m3m3(rho_oil_st_kgm3=820, rs_m3m3=100, rho_oil_insitu_kgm3=700, gamma_gas=0.8):
    """
        Oil Formation Volume Factor according McCain correlation for pressure below bubble point pressure

    :param rho_oil_st_kgm3: density of stock-tank oil, kgm3
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param rho_oil_insitu_kgm3: Oil density at reservoir conditions, kgm3
    :param gamma_gas: specific gas  density(by air)
    :return: formation volume factor bo, m3m3

    ref1 book Mccain_w_d_spivey_j_p_lenn_c_p_petroleum_reservoir_fluid,third edition, 2011
    """
    # коэффициенты преобразованы - смотри описанию в ноутбуке
    bo = (rho_oil_st_kgm3 + 1.22044505587208 * rs_m3m3 * gamma_gas) / rho_oil_insitu_kgm3
    return bo

def unf_bo_saturated_Standing_m3m3(rs_m3m3=100, gamma_gas=0.8, gamma_oil=0.86, t_K=300):
    """
        Oil Formation Volume Factor according Standing equation at bubble point pressure

    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param gamma_gas: specific gas density (by air)
    :param gamma_oil: specific oil density (by water)
    :param t_K: temperature, K
    :return: formation volume factor at bubble point pressure bo,m3m3

    ref1 Volumetric and phase behavior of oil field hydrocarbon systems / M.B. Standing Standing, M. B. 1981
    """

    rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
    t_F = uc.K_2_F(t_K)
    bo = 0.972 + 1.47e-4 * (rs_scfstb * (gamma_gas / gamma_oil) ** 0.5 + 1.25 * t_F) ** 1.175
    return bo


""" 
====================================================================================================
Корреляции расчета плотности нефти
====================================================================================================
"""

def unf_density_oil_Mccain(p_MPaa=1, pb_MPaa=10, co_1MPa=3e-3, rs_m3m3=10, gamma_gas=0.86, t_K=300, gamma_oil=0.86, gamma_gassp = 0):
    """
        Oil density according Standing, M.B., 1977; Witte, T.W., Jr., 1987; and McCain, W.D., Jr. and Hill, N.C.,
    1995 correlation.

    :param p_MPaa: pressure, MPaa
    :param pb_MPaa: bubble point pressure, MPaa
    :param co_1MPa: coefficient of isothermal compressibility, 1/MPa
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :param gamma_oil: specific oil density (by water)
    :param gamma_gassp: specific gas density in separator(by air)
    :return: oil density,kg/m3

    ref1 book Mccain_w_d_spivey_j_p_lenn_c_p_petroleum_reservoir_fluid,third edition, 2011
    """
    rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
    t_F = uc.K_2_F(t_K)
    p_psia = uc.MPa_2_psi(p_MPaa)

    gamma_gassp = np.where(gamma_gassp == 0, gamma_gas, gamma_gassp)

    def ro_po_equation(ro_po, vars):
        gamma_gassp, gamma_gas, gamma_oil, rs_scfstb = vars
        ro_a = -49.8930 + 85.0149 * gamma_gassp - 3.70373 * gamma_gassp * ro_po +\
            0.0479818 * gamma_gassp * ro_po ** 2 + 2.98914 * ro_po - 0.0356888 * ro_po ** 2
        return (rs_scfstb * gamma_gas + 4600 * gamma_oil) / (73.71 + rs_scfstb * gamma_gas / ro_a) -ro_po
    
    ro_po = 52.8 - 0.01 * rs_scfstb  # первое приближение
    ro_po = opt.fsolve(ro_po_equation, ro_po, [gamma_gassp, gamma_gas, gamma_oil, rs_scfstb])
    pb_psia = uc.MPa_2_psi(pb_MPaa)
    p = np.where(p_MPaa < pb_MPaa, p_psia, pb_psia)

    dro_p = ((0.167 + 16.181 * 10 ** (-0.0425 * ro_po)) * (p / 1000) - 0.01 * 
             (0.299 + 263 * 10 ** (-0.0603 * ro_po)) * (p / 1000) ** 2)
    ro_bs = ro_po + dro_p
    dro_t = ((0.00302 + 1.505 * ro_bs ** (-0.951)) * (t_F - 60) ** 0.938 - 
             (0.0216 - 0.0233 * 10 ** (-0.0161 * ro_bs)) * (t_F - 60) ** 0.475)
    ro_or = ro_bs - dro_t

    ro_or = np.where(p_MPaa >= pb_MPaa, 
                     ro_or * np.exp(uc.compr_1pa_2_1psi(co_1MPa / 1e6) * (p_psia - pb_psia)),
                     ro_or)
    ro_or = uc.lbft3_2_kgm3(ro_or)
    return ro_or


    
def unf_density_oil_Standing(p_MPaa=1, pb_MPaa=10, co_1MPa=3e-3, rs_m3m3=10, bo_m3m3=1.1, gamma_gas=0.8, gamma_oil=0.86):
    """
        Oil density according Standing correlation.

    :param p_MPaa: pressure, MPaa
    :param pb_MPaa: bubble point pressure, MPaa
    :param co_1MPa: coefficient of isothermal compressibility, 1/MPa
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param bo_m3m3: oil formation volume factor, m3m3
    :param gamma_gas: specific gas density (by air)
    :param gamma_oil: specific oil density (by water)
    :return: oil density,kg/m3

    ref1 book Brill 2006, Production Optimization Using Nodal Analysis
    """
    po = (1000 * gamma_oil + 1.224 * gamma_gas * rs_m3m3) / bo_m3m3

    # TODO возможно это надо оставить, при давлении выше P нас, уже есть в fvf_oil степень с co
        
    return np.where(p_MPaa > pb_MPaa, po * np.exp(co_1MPa * (p_MPaa - pb_MPaa)), po)


""" 
====================================================================================================
Корреляции расчета сжимаемости нефти
====================================================================================================
"""
def unf_compressibility_saturated_oil_McCain_1Mpa(p_MPa=1, pb_MPa=10, t_K=300, gamma_oil=0.86, rsb_m3m3=100):
    """
        Oil compressibility below bubble point (saturated oil)

    :param p_mpa:
    :param pb_mpa:
    :param t_k:
    :param gamma_oil:
    :param rsb_m3m3:
    :return:

    ref1 https://www.researchgate.net/publication/
    254529353_The_Oil_Compressibility_Below_Bubble_Point_Pressure_Revisited_-_Formulations_and_Estimations

    ref2 https://www.onepetro.org/download/journal-paper/SPE-15664-PA?id=journal-paper%2FSPE-15664-PA
    """
    rsb_scfstb = uc.m3m3_2_scfstb(rsb_m3m3)
    t_F = uc.K_2_F(t_K)
    api = uc.gamma_oil_2_api(gamma_oil)
    p_psia = uc.MPa_2_psi(p_MPa)
    pb_psia = uc.MPa_2_psi(pb_MPa)
    co_1psi = np.exp(-7.573 - 1.450 * np.log(p_psia) - 0.383 * np.log(pb_psia) + 1.402 * np.log(t_F) +
                     0.256 * np.log(api) + 0.449 * np.log(rsb_scfstb))
    co_1MPa = uc.compr_1psi_2_1MPa(co_1psi)# * 10 ** 5 # TODO надо разобраться и исправить
    return co_1MPa


def unf_compressibility_oil_VB_1Mpa(rs_m3m3, t_K, gamma_oil, p_MPaa, gamma_gas=0.6): # TODO above bubble point!!
    """
        Oil compressibility according to Vasquez & Beggs (1980) correlation

    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param t_K: temperature, K
    :param gamma_oil: specific oil density (by water)
    :param p_MPaa: pressure, MPaa
    :param gamma_gas: specific gas density (by air)
    :return: coefficient of isothermal compressibility co_1MPa, 1/MPa

    ref1 Vazquez, M. and Beggs, H.D. 1980. Correlations for Fluid Physical Property Prediction.
    J Pet Technol 32 (6): 968-970. SPE-6719-PA

    """

    rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
    t_F = uc.K_2_F(t_K)
    api = uc.gamma_oil_2_api(gamma_oil)
    p_psia = uc.MPa_2_psi(p_MPaa)
    
    return np.where(p_MPaa > 0, 
                    uc.compr_1psi_2_1MPa( (-1433 + 5 * rs_scfstb + 17.2 * t_F - 1180 * gamma_gas + 12.61 * api) / (1e5 * p_psia)),
                    0.0
                    )

def unf_compessibility_oil_VB_1MPa_vba(rsb_m3m3, t_k, gamma_oil, gamma_gas):
    co_1atm = (28.1 * rsb_m3m3 + 30.6 * t_k - 1180 * gamma_gas + 1784 / gamma_oil - 10910)
    return 1/uc.atm_2_bar(1/co_1atm)/10  #*))) **когда не дружишь с математикой ***когда слишком много времени потратил на uniflocpy

""" 
====================================================================================================
Корреляции расчета плотности газа на поверхности
====================================================================================================
"""
def unf_gamma_gas_Mccain(rsp_m3m3, rst_m3m3, gamma_gassp=0.8, gamma_oil=0.86, psp_MPaa=0, tsp_K=0):
    """
        Correlation for weighted-average specific gravities of surface gases

    :param rsp_m3m3: separator producing gas-oil ratio, m3m3
    :param rst_m3m3: stock-tank producing gas-oil ratio, m3m3
    :param gamma_gassp:  separator gas specific gravity
    :param gamma_oil: specific oil density(by water)
    :param psp_MPaa: pressure in separator, MPaa
    :param tsp_K: temperature in separator, K
    :return: weighted-average specific gravities of surface gases

    часто условия в сепараторе неизвестны, может считать и без них по приблизительной формуле

    ref1 "Reservoir oil bubblepoint pressures revisited; solution gas–oil ratios and surface gas specific gravities",
    J. VELARDE, W.D. MCCAIN, 2002

    """

    api = uc.gamma_oil_2_api(gamma_oil)
    psp_psia = uc.MPa_2_psi(psp_MPaa)
    tsp_F = uc.K_2_F(tsp_K)
    rsp_scfstb = uc.m3m3_2_scfstb(rsp_m3m3)
    rst_scfstb = uc.m3m3_2_scfstb(rst_m3m3)

    nplogpsp_psia = np.log(psp_psia) 
    nplogrsp_scfstb = np.log(rsp_scfstb)

    z1 = -17.275 + 7.9597 * nplogpsp_psia - 1.1013 * nplogpsp_psia ** 2 + 2.7735e-2 \
        * nplogpsp_psia ** 3 + 3.2287e-3 * nplogpsp_psia ** 4
    z2 = -0.3354 - 0.3346 * nplogrsp_scfstb + 0.1956 * nplogrsp_scfstb ** 2 - 3.4374e-2 \
        * nplogrsp_scfstb ** 3 + 2.08e-3 * nplogrsp_scfstb ** 4
    z3 = 3.705 - 0.4273 * api + 1.818e-2 * api ** 2 - 3.459e-4 \
        * api ** 3 + 2.505e-6 * api ** 4
    z4 = -155.52 + 629.61 * gamma_gassp - 957.38 * gamma_gassp ** 2 + 647.57 \
        * gamma_gassp ** 3 - 163.26 * gamma_gassp ** 4
    z5 = 2.085 - 7.097e-2 * tsp_F + 9.859e-4 * tsp_F ** 2 \
        - 6.312e-6 * tsp_F ** 3 + 1.4e-8 * tsp_F ** 4
    z = z1 + z2 + z3 + z4 + z5
    # Stock-tank gas specific gravity
    gamma_gasst = 1.219 + 0.198 * z + 0.0845 * z ** 2 + 0.03 * z ** 3 + 0.003 * z ** 4
      
    return np.where(psp_MPaa * rsp_m3m3 * rst_m3m3 * gamma_gassp > 0,
                    (gamma_gassp * rsp_scfstb + gamma_gasst * rst_scfstb) / (rsp_scfstb + rst_scfstb),
                    np.where(gamma_gassp >= 0,
                            1.066 * gamma_gassp,
                            0
                            )
                    )


""" 
====================================================================================================
Корреляции расчета вязкости нефти
====================================================================================================
"""


def unf_deadoilviscosity_Beggs_cP(gamma_oil, t_K):
    """
        Correlation for dead oil viscosity

    :param gamma_oil: specific oil density (by water)
    :param t_K: temperature, K
    :return: dead oil viscosity,cP

    ref1 Beggs, H.D. and Robinson, J.R. “Estimating the Viscosity of Crude Oil Systems.”
    Journal of Petroleum Technology. Vol. 27, No. 9 (1975)

    """
    api = uc.gamma_oil_2_api(gamma_oil)
    t_F = uc.K_2_F(t_K)
    c = 10 ** (3.0324 - 0.02023 * api) * t_F ** (-1.163)  
    return 10 ** c - 1

def unf_saturatedoilviscosity_Beggs_cP(deadoilviscosity_cP, rs_m3m3):
    """
        Correlation for oil viscosity for pressure below bubble point (for pb!!!)

    :param deadoilviscosity_cP: dead oil viscosity,cP
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :return: oil viscosity,cP

    ref1 Beggs, H.D. and Robinson, J.R. “Estimating the Viscosity of Crude Oil Systems.”
    Journal of Petroleum Technology. Vol. 27, No. 9 (1975)

    """
    rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
    a = 10.715 * (rs_scfstb + 100) ** (-0.515)
    b = 5.44 * (rs_scfstb + 150) ** (-0.338)
    viscosity_cP = a * deadoilviscosity_cP ** b
    return viscosity_cP


def unf_undersaturatedoilviscosity_VB_cP(p_MPaa, pb_MPaa, bubblepointviscosity_cP):
    """
        Viscosity correlation for pressure above bubble point

    :param p_MPaa: pressure, MPaa
    :param pb_MPaa: bubble point pressure, MPaa
    :param bubblepointviscosity_cP: oil viscosity at bubble point pressure, cP
    :return: oil viscosity,cP

    ref2 Vazquez, M. and Beggs, H.D. 1980. Correlations for Fluid Physical Property Prediction.
    J Pet Technol 32 (6): 968-970. SPE-6719-PA

    """
    p_psia = uc.MPa_2_psi(p_MPaa)
    pb_psia = uc.MPa_2_psi(pb_MPaa)
    m = 2.6 * p_psia ** 1.187 * np.exp(-11.513 - 8.98e-5 * p_psia)
    viscosity_cP = bubblepointviscosity_cP * (p_psia / pb_psia) ** m
    return viscosity_cP

def unf_oil_viscosity_Beggs_VB_cP(deadoilviscosity_cP, rs_m3m3, p_MPaa, pb_MPaa):
    """
        Function for calculating the viscosity at any pressure

    :param deadoilviscosity_cP: dead oil viscosity,cP
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param p_MPaa: pressure, MPaa
    :param pb_MPaa: bubble point pressure, MPaa
    :return: oil viscosity,cP
    """
    
    saturatedviscosity_cP = unf_saturatedoilviscosity_Beggs_cP(deadoilviscosity_cP, rs_m3m3)
    return np.where(p_MPaa <= pb_MPaa, 
                    saturatedviscosity_cP,
                    unf_undersaturatedoilviscosity_VB_cP(p_MPaa, pb_MPaa, saturatedviscosity_cP)
                    )


def unf_heat_capacity_oil_Gambill_JkgC(gamma_oil, t_C):
    """
        Oil heat capacity in SI. Gambill correlation

    :param gamma_oil: specific oil density(by water)
    :param t_c: temperature in C
    :return: heat capacity in SI - JkgC

    ref1 Book: Brill J. P., Mukherjee H. K. Multiphase flow in wells. –
    Society of Petroleum Engineers, 1999. – Т. 17. in Page 122
    """

    t_F = uc.C_2_F(t_C)
    heat_capacity_oil_btulbmF = ((0.388 + 0.00045 * t_F) / gamma_oil ** (0.5))
    return uc.btulbmF_2_kJkgK(heat_capacity_oil_btulbmF) * 1000


def unf_thermal_conductivity_oil_Cragoe_WmK(gamma_oil, t_C):
    """
        Oil thermal conductivity Cragoe correlation for 273 < T < 423 K

    :param gamma_oil: specific oil density(by water)
    :param t_c: temperature in C
    :return: thermal conductivity in SI - wt / m K

    ref1 Das D. K., Nerella S., Kulkarni D. Thermal properties of petroleum and gas-to-liquid products //
    Petroleum science and technology. – 2007. – Т. 25. – №. 4. – С. 415-425.   """
    t_K = uc.C_2_K(t_C)
    return (0.118 /(gamma_oil * 1000) * (1 - 0.00054 * (t_K - 273)) * 10 ** 3)


#from pyrsistent import T
import neftpy.upvt as pvt
import neftpy.upvt_old as pvt_old
import numpy as np

""" 
p_MPaa =  np.array([0.1, 1,5,10, 20, 30])
pb_MPaa = 10
co_1MPa = 3 * 10**(-3)
rs_m3m3 = 150
gamma_gas = 0.6
t_K = 350
gamma_oil = 0.86
gamma_gassp = 0
res = pvt.unf_density_oil_Mccain(p_MPaa, pb_MPaa, co_1MPa, rs_m3m3, gamma_gas, t_K, gamma_oil,gamma_gassp)

print(res)

res = pvt_old.unf_density_oil_Mccain(p_MPaa, pb_MPaa, co_1MPa, rs_m3m3, gamma_gas, t_K, gamma_oil,gamma_gassp)

print(res)
"""

import neftpy.fluid as fl 

f = fl.BlackOilStanding(gamma_gas=0.6, gamma_oil=0.82)
f.set_calibration_param(pb_calibr_bar=100, tb_calibr_C=80, b_oilb_calibr_m3m3=1.1)

p = np.linspace(1, 100, 10)
f.calc(p_bar=p, t_C= 90)

print(f.pb_bar)
print(f.rs_m3m3)
print(f.b_oil_m3m3)


import unittest
import neftpy.upvt as pvt
import neftpy.upvt_old as pvt_old
import numpy as np

#from ..neftpy.upvt import *

class PVTTestCase(unittest.TestCase):

    """
    Расчет давления насыщения по Стендингу
    """
    def test_unf_pb_Standing_MPaa(self):
        rsb_m3m3 = 100
        gamma_oil = 0.86
        gamma_gas = 0.6
        t_K = 350
        self.assertAlmostEqual(pvt.unf_pb_Standing_MPaa(rsb_m3m3, gamma_oil, gamma_gas, t_K), 20.170210695316566,
                               delta=0.0001)

    def test_array_unf_pb_Standing_MPaa(self):
        """ check with np.array as input """
        rsb_m3m3 = np.array([1.0, 5.0, 10.0, 100.0, 200.0]) 
        gamma_oil = 0.86
        gamma_gas = 0.6
        t_K = 350
        res = np.array([ 0.44433583,  1.67824568,  2.98339278, 20.1702107 , 35.85628831])
        self.assertTrue(np.allclose(pvt.unf_pb_Standing_MPaa(rsb_m3m3, gamma_oil, gamma_gas, t_K), res))
    
    def test_array_2_unf_pb_Standing_MPaa(self):
        """ compares array input for vectorised realisations """
        rsb_m3m3 = np.array([1.0, 5.0, 10.0, 100.0, 200.0]) 
        gamma_oil = 0.86
        gamma_gas = 0.6
        t_K = 350
        res = pvt_old.unf_pb_Standing_MPaa(rsb_m3m3, gamma_oil, gamma_gas, t_K)  # naive vectorisation
        self.assertTrue(np.allclose(pvt.unf_pb_Standing_MPaa(rsb_m3m3, gamma_oil, gamma_gas, t_K), res))

    """
    Расчет давления насыщения по Валко МакКейну
    """
    def test_unf_pb_Valko_MPaa(self):
        rsb_m3m3 = 100
        gamma_oil = 0.86
        gamma_gas = 0.6
        t_K = 350
        self.assertAlmostEqual(pvt.unf_pb_Valko_MPaa(rsb_m3m3, gamma_oil, gamma_gas, t_K), 23.29991481380937,
                               delta=0.00001)

    def test_array_unf_pb_Valko_MPaa(self):
        """ compares array input for vectorised realisations """
        rsb_m3m3 = np.array([1.0, 5.0, 10.0, 100.0, 200.0]) 
        gamma_oil = 0.86
        gamma_gas = 0.6
        t_K = 350
        res = pvt_old.unf_pb_Valko_MPaa(rsb_m3m3, gamma_oil, gamma_gas, t_K)  # naive vectorisation
        self.assertTrue(np.allclose(pvt.unf_pb_Valko_MPaa(rsb_m3m3, gamma_oil, gamma_gas, t_K), res))

    def test_unf_rs_Standing_m3m3(self):
        p_MPaa = 10
        pb_MPaa = 15
        rsb = 200
        gamma_oil = 0.86
        gamma_gas = 0.6
        t_K = 350
        self.assertAlmostEqual(pvt.unf_rs_Standing_m3m3(p_MPaa, pb_MPaa, rsb, gamma_oil, gamma_gas, t_K),
                               122.74847910146916, delta=0.0001)

    def test_array_unf_rs_Standing_m3m3(self):
        p_MPaa = np.array([1.0, 5.0, 10.0, 100.0, 200.0]) 
        pb_MPaa = 15
        rsb = np.array([1.0, 5.0, 10.0, 100.0, 200.0]) 
        gamma_oil = 0.86
        gamma_gas = 0.6
        t_K = 350
        self.assertTrue(np.allclose(pvt.unf_rs_Standing_m3m3(p_MPaa, pb_MPaa, rsb, gamma_oil, gamma_gas, t_K),
                                    pvt_old.unf_rs_Standing_m3m3(p_MPaa, pb_MPaa, rsb, gamma_oil, gamma_gas, t_K)
                                    ) 
                        )

    def test_unf_rs_Velarde_m3m3(self):
        p_MPaa = 10
        pb_MPaa = 15
        rsb = 250
        gamma_oil = 0.86
        gamma_gas = 0.6
        t_K = 350
        self.assertAlmostEqual(pvt.unf_rs_Velarde_m3m3(p_MPaa, pb_MPaa, rsb, gamma_oil, gamma_gas, t_K),
                               170.25302712587356, delta=0.0001)

    def test_array_unf_rs_Velarde_m3m3(self):
        p_MPaa = np.array([1.0, 5.0, 10.0, 100.0, 200.0]) 
        pb_MPaa = 15
        rsb = np.array([1.0, 5.0, 10.0, 100.0, 200.0]) 
        gamma_oil = 0.86
        gamma_gas = 0.6
        t_K = 350
        self.assertTrue(np.allclose(pvt.unf_rs_Velarde_m3m3(p_MPaa, pb_MPaa, rsb, gamma_oil, gamma_gas, t_K),
                                    pvt_old.unf_rs_Velarde_m3m3(p_MPaa, pb_MPaa, rsb, gamma_oil, gamma_gas, t_K)
                                    ) 
                        )
                        
    def test_unf_rsb_Mccain_m3m3(self):
        rsp_m3m3 = 150
        gamma_oil = 0.86
        psp_MPaa = 5
        tsp_K = 320
        self.assertAlmostEqual(pvt.unf_rsb_Mccain_m3m3(rsp_m3m3, gamma_oil, psp_MPaa, tsp_K),
                               161.03286985548442, delta=0.0001)

                        
    def test_array_unf_rsb_Mccain_m3m3(self):
        rsp_m3m3 = np.linspace(1,100,20)
        gamma_oil = 0.86
        psp_MPaa = 5
        tsp_K = 320
        self.assertTrue(np.allclose(pvt_old.unf_rsb_Mccain_m3m3(rsp_m3m3, gamma_oil, psp_MPaa, tsp_K),
                                    pvt.unf_rsb_Mccain_m3m3(rsp_m3m3, gamma_oil, psp_MPaa, tsp_K)
                                    ) 
                        )
        
    def test_unf_bo_above_pb_m3m3(self):
        bob = 1.3
        cofb_1MPa = 3e-3
        pb_MPaa = 12
        p_MPaa = 15
        self.assertAlmostEqual(pvt_old.unf_bo_above_pb_m3m3(bob, cofb_1MPa, pb_MPaa, p_MPaa), 1.2883524924047487, delta=0.0001)

    def test_array_unf_bo_above_pb_m3m3(self):
        bob = 1.3
        cofb_1MPa = 3e-3
        pb_MPaa = 12
        p_MPaa = np.linspace(0.1,10,20)
        self.assertTrue(np.allclose(pvt_old.unf_bo_above_pb_m3m3(bob, cofb_1MPa, pb_MPaa, p_MPaa),
                                    pvt.unf_bo_above_pb_m3m3(bob, cofb_1MPa, pb_MPaa, p_MPaa)
                                    ) 
                        )

    def test_unf_bo_below_m3m3(self):
        density_oilsto_kgm3 = 800
        rs_m3m3 = 200
        density_oil_kgm3 = 820
        gamma_gas = 0.6
        self.assertAlmostEqual(pvt.unf_bo_below_pb_m3m3(density_oilsto_kgm3, rs_m3m3, density_oil_kgm3, gamma_gas),
                               1.1542114715227887, delta=0.0001
                               )

    def test_array_unf_bo_below_m3m3(self):
        density_oilsto_kgm3 = 800
        rs_m3m3 = np.linspace(1,100,20)
        density_oil_kgm3 = 820
        gamma_gas = 0.6
        self.assertTrue(np.allclose(pvt_old.unf_bo_below_pb_m3m3(density_oilsto_kgm3, rs_m3m3, density_oil_kgm3, gamma_gas),
                                    pvt.unf_bo_below_pb_m3m3(density_oilsto_kgm3, rs_m3m3, density_oil_kgm3, gamma_gas)
                                    ) 
                        )


    def test_unf_density_oil_Mccain(self):
        p_MPaa = 10
        pb_MPaa = 12
        co_1MPa = 3 * 10**(-3)
        rs_m3m3 = 250
        gamma_gas = 0.6
        t_K = 350
        gamma_oil = 0.86
        gamma_gassp = 0
        self.assertAlmostEqual(pvt.unf_density_oil_Mccain(p_MPaa, pb_MPaa, co_1MPa, rs_m3m3, gamma_gas, t_K, gamma_oil,
                               gamma_gassp), 630.0536681794456, delta=0.0001)


    def test_array_unf_density_oil_Mccain(self):
        p_MPaa = np.array([0.1, 1,5,10, 20, 30])
        pb_MPaa = 10
        co_1MPa = 3 * 10**(-3)
        rs_m3m3 = 150
        gamma_gas = 0.6
        t_K = 350
        gamma_oil = 0.86
        gamma_gassp = 0
        self.assertTrue(np.allclose(pvt.unf_density_oil_Mccain(p_MPaa, pb_MPaa, co_1MPa, rs_m3m3, gamma_gas, t_K, gamma_oil,
                               gamma_gassp),
                               [684.20874678, 684.97844157, 688.3460601,  692.43351253, 713.52125245, 735.25120966]) )
  
    def test_unf_density_oil_Standing(self):
        p_MPaa = 10
        pb_MPaa = 12
        co_1MPa = 3 * 10**(-3)
        rs_m3m3 = 250
        bo_m3m3 = 1.1
        gamma_gas = 0.6
        gamma_oil = 0.86
        self.assertAlmostEqual(pvt.unf_density_oil_Standing(p_MPaa, pb_MPaa, co_1MPa, rs_m3m3, bo_m3m3, gamma_gas, gamma_oil
                                                        ), 948.7272727272725, delta=0.0001)

    #def test_unf_compressibility_saturated_oil_VB_1Mpa(self):
    #    rsb_m3m3 = 200
    #    t_k = 350
    #    gamma_oil = 0.86
    #    p_MPaa = 10
    #    pb_mpa = 15
    #    self.assertAlmostEqual(pvt.unf_compressibility_saturated_oil_McCain_1Mpa(p_MPaa, pb_mpa, t_k, gamma_oil, rsb_m3m3),
    #                           0.00078861, delta=0.0001)
    #  тест пока работает неправильно, надо отдельно разобраться с расчетом сжимаемостей

    def test_unf_compressibility_oil_VB_1Mpa(self):
        rs_m3m3 = 200
        t_K = 350
        gamma_oil = 0.86
        p_MPaa = 15
        gamma_gas = 0.6
        self.assertAlmostEqual(pvt.unf_compressibility_oil_VB_1Mpa(rs_m3m3, t_K, gamma_oil, p_MPaa, gamma_gas),
                               0.004546552811369566, delta=0.0001)


    def test_unf_gamma_gas_Mccain(self):
        rsp_m3m3 = 30
        rst_m3m3 = 20
        gamma_gassp = 0.65
        gamma_oil = 0.86
        psp_MPaa = 5
        tsp_K = 350
        self.assertAlmostEqual(pvt.unf_gamma_gas_Mccain(rsp_m3m3, rst_m3m3, gamma_gassp, gamma_oil, psp_MPaa, tsp_K),
                               0.7932830162938984, delta=0.0001)




# Executing the tests in the above test case class
if __name__ == "__main__":
  unittest.main()


"""
Constant and units conversion functions for for petroleum engineering calculations
"""
import numpy as np
import scipy.constants as const


g = const.g   # gravity
pi = const.pi
pressure_sc_bar = 1   # pressure standard condition
temperature_sc_C = 15    # temperature standard condition
const_at = 98066.5  # technical atmosphere in Pa,  техническая атмосфера в Па
air_density_sc_kgm3 = 1.225  # definition from https://en.wikipedia.org/wiki/Density_of_air
rho_water_sc_kgm3 = 1000

# some default values 
default_z = 0.9
default_gamma_gas = 0.8
default_rsb_m3m3 = 100


# simple unit conversion functions

# pressure

def psi_2_Pa(value):
    """
    converts pressure in psi (pound-force per square inch) to Pa (Pascal)
    :param value: pressure value in psi
    :return: pressure value in Pa
    """
    return value * const.psi


def Pa_2_psi(value):
    """
    converts pressure in Pa (Pascal) to psi (pound-force per square inch)
    :param value: pressure value in Pa
    :return: pressure value in psi
    """
    return value / const.psi

def Pa_2_psig(value):
    """
    converts pressure in abs Pa (Pascal) to gauged psig (pound-force per square inch)
    :param value: pressure value in Pa
    :return: pressure value in psi
    """
    return value / const.psi - 14.7

def MPa_2_psi(value):
    """
    converts pressure in MPa (Mega Pascal, 1e6 Pa) to psi (pound-force per square inch)
    :param value: pressure value in MPa
    :return: pressure value in Pa
    """
    return value * const.mega / const.psi

def MPa_2_psig(value):
    """
    converts pressure 
    in absolute MPa (Mega Pascal, 1e6 Pa) to measured psig (pound-force per square inch)
    :param value: pressure value in MPa
    :return: pressure value in Pa
    """
    return value * const.mega / const.psi - 14.7

def bar_2_Pa(value):
    """
    converts pressure in bar (1e5 Pa) to Pa
    :param value: pressure value in bar
    :return: pressure value in Pa
    """
    return value * const.bar

def bar_2_MPa(value):
    """
    converts pressure in bar to Pa
    :param value: pressure value in bar
    :return: pressure value in Pa
    """
    return value * const.bar / const.mega


def Pa_2_bar(value):
    """
    converts pressure in Pa to bar
    :param value: pressure value in Pa
    :return: pressure value in bar
    """
    return value / const.bar

def MPa_2_bar(value):
    """
    converts pressure in MPa to bar
    :param value: pressure value in MPa
    :return: pressure value in bar
    """
    return value * const.mega / const.bar

def atm_2_Pa(value):
    """
    converts pressure in atm (standard atmosphere) to Pa
    :param value: pressure value in atm
    :return: pressure value in Pa
    """
    return value * const.atm


def Pa_2_atm(value):
    """
    converts pressure in Pa to atm (standard atmosphere)
    :param value: pressure value in Pa
    :return: pressure value in atm (standard atmosphere)
    """
    return value / const.atm


def at_2_Pa(value):
    """
    converts pressure in at (technical atmosphere) to Pa
    :param value: pressure value in at (technical atmosphere)
    :return: pressure value in Pa
    """
    return value * const_at


def Pa_2_at(value):
    """
    converts pressure in Pa (Pascal) to at (technical atmosphere)
    :param value: pressure value in Pa
    :return: pressure value in at (technical atmosphere)
    """
    return value * const_at


def bar_2_psi(value):
    """
    converts pressure in bar to psi
    :param value: pressure value in bar
    :return: pressure value in psi
    """
    return value * const.bar / const.psi


def psi_2_bar(value):
    """
     converts pressure in psi to bar
     :param value: pressure value in psi
     :return: pressure value in bar
     """
    return value / const.bar * const.psi


def bar_2_atm(value=1):
    """
     converts pressure in bar to atm(standard atmosphere)
     :param value: pressure value in bar
     :return: pressure value in atm(standard atmosphere)
     """
    return value * const.bar / const.atm


def atm_2_bar(value=1):
    """
     converts pressure in atm(standard atmosphere) to bar
     :param value: pressure value in atm(standard atmosphere)
     :return: pressure value in bar
     """
    return value * const.atm / const.bar

# temperature


def C_2_F(value):
    """
     converts temperature in C(degrees Celsius) to F(degrees Fahrenheit)
     :param value: temperature in C(degrees Celsius)
     :return: temperature in F(degrees Fahrenheit)
     """
    return const.convert_temperature(value, 'C', 'F')


def F_2_C(value):
    """
     converts temperature in F(degrees Fahrenheit) to C(degrees Celsius)
     :param value: temperature in F(degrees Fahrenheit)
     :return: temperature in C(degrees Celsius)
     """
    return const.convert_temperature(value, 'F', 'K')


def C_2_K(value):
    """
     converts temperature in C(degrees Celsius) to K(Kelvins)
     :param value: temperature in C(degrees Celsius)
     :return: temperature in K(Kelvins)
     """
    return const.convert_temperature(value, 'C', 'K')


def K_2_C(value):
    """
     converts temperature in K(Kelvins) to C(degrees Celsius)
     :param value: temperature in K(Kelvins)
     :return: temperature in C(degrees Celsius)
     """
    return const.convert_temperature(value, 'K', 'C')


def K_2_F(value):
    """
     converts temperature in F(degrees Fahrenheit) to K(Kelvins)
     :param value: temperature in F(degrees Fahrenheit)
     :return: temperature in K(Kelvins)
     """
    return const.convert_temperature(value, 'K', 'F')


def F_2_K(value):
    """
     converts temperature in F(degrees Fahrenheit) to K(Kelvins)
     :param value: temperature in F(degrees Fahrenheit)
     :return: temperature in K(Kelvins)
     """
    return const.convert_temperature(value, 'F', 'K')


def F_2_R(value):
    """
     converts temperature in F(degrees Fahrenheit) to R(degrees Rankine)
     :param value: temperature in F(degrees Fahrenheit)
     :return: temperature in R(degrees Rankine)
     """
    return const.convert_temperature(value, 'F', 'R')


def R_2_F(value):
    """
     converts temperature in R(degrees Rankine) to F(degrees Fahrenheit)
     :param value: temperature in R(degrees Rankine)
     :return: temperature in F(degrees Fahrenheit)
     """
    return const.convert_temperature(value, 'R', 'F')


def C_2_R(value):
    """
     converts temperature in C(degrees Celsius) to R(degrees Rankine)
     :param value: temperature in C(degrees Celsius)
     :return: temperature in R(degrees Rankine)
     """
    return const.convert_temperature(value, 'C', 'R')


def R_2_C(value):
    """
     converts temperature in R(degrees Rankine) to C(degrees Celsius)
     :param value: temperature in C(degrees Celsius)
     :return: temperature in R(degrees Rankine)
     """
    return const.convert_temperature(value, 'R', 'C')


def K_2_R(value):
    """
     converts temperature in K(Kelvins) to R(degrees Rankine)
     :param value: temperature in K(Kelvins)
     :return: temperature in R(degrees Rankine)
     """
    return const.convert_temperature(value, 'K', 'R')


def R_2_K(value):
    """
     converts temperature in R(degrees Rankine) to K(Kelvins)
     :param value: temperature in R(degrees Rankine)
     :return: temperature in K(Kelvins)
     """
    return const.convert_temperature(value, 'R', 'K')

# length


def m_2_in(value):
    """
    converts length in m(meters) to in(inches)
    :param value: length in m(meters)
    :return: length in in(inches)
    """
    return value / const.inch


def in_2_m(value):
    """
    converts length in in(inches) to m(meters)
    :param value: length in in(inches)
    :return: length in m(meters)
    """
    return value * const.inch


def m_2_ft(value):
    """
    converts length in m(meters) to ft(feet)
    :param value: length in m(meters)
    :return: length in ft(feet)
    """
    return value / const.foot


def ft_2_m(value):
    """
    converts length in ft(feet) to m(meters)
    :param value: length in ft(feet)
    :return: length in m(meters)
    """
    return value * const.foot

# volume


def m3_2_bbl(value):
    """
     converts volume in m3(cubic metres) to bbl(barrels)
     :param value: volume in m3(cubic metres)
     :return: volume in bbl(barrels)
     """
    return value / const.barrel


def bbl_2_m3(value):
    """
     converts volume in m3(cubic metres) to bbl(barrels)
     :param value: volume in m3(cubic metres)
     :return: volume in bbl(barrels)
     """
    return value * const.barrel

# GOR


def m3m3_2_m3t(value, gamma=1):
    """
     converts Gas-Oil Ratio in m3/m3(cubic metres/cubic meter) to m3/t(cubic metres/ton)
     :param value: Gas-Oil Ratio in m3/m3(cubic metres/cubic meter);
            gamma=1: oil density(by air)
     :return: Gas-Oil Ratio in m3/t(cubic metres/ton)
     """
    return value * gamma


def m3t_2_m3m3(value, gamma=1):
    """
     converts Gas-Oil Ratio in m3/t(cubic metres/ton) to m3/m3(cubic metres/cubic meter)
     :param value: Gas-Oil Ratio in m3/t(cubic metres/ton);
            gamma=1: oil density(by air)
     :return: Gas-Oil Ratio in m3/m3(cubic metres/cubic meter)
     """
    return value / gamma


def scfstb_2_m3m3(value):
    """
     converts Gas-Oil Ratio in scf/stb(standard cubic feet/standard barrel) to m3/m3(cubic metres/cubic meter)
     :param value: Gas-Oil Ratio in scf/stb(standard cubic feet/standard barrel)
     :return: Gas-Oil Ratio in m3/m3(cubic metres/cubic meter)
     """
    return value * (const.foot ** 3 / const.bbl)


def m3m3_2_scfstb(value):
    """
     converts Gas-Oil Ratio in m3/m3(cubic metres/cubic meter) to scf/stb(standard cubic feet/standard barrel)
     :param value: Gas-Oil Ratio in m3/m3(cubic metres/cubic meter)
     :return: Gas-Oil Ratio in scf/stb(standard cubic feet/standard barrel)
     """
    return value / (const.foot ** 3 / const.bbl)


# density


def api_2_gamma_oil(value):
    """
     converts density in API(American Petroleum Institute gravity) to gamma_oil (oil relative density by water)
     :param value: density in API(American Petroleum Institute gravity)
     :return: oil relative density by water
     """
    return (value + 131.5) / 141.5


def gamma_oil_2_api(value):
    """
     converts density in API(American Petroleum Institute gravity) to gamma_oil (oil relative density by water)
     :param value: oil relative density by water
     :return: density in API(American Petroleum Institute gravity)
     """
    return 141.5 / value - 131.5


def kgm3_2_lbft3(value):
    """
     converts density in kg/m3(kilogrammes/cubic meter) to lb/ft3(pounds/cubic feet)
     :param value: density in kgm3(kilogrammes/cubic meter)
     :return: density in lb/ft3(pounds/cubic feet)
     """
    return value * (const.foot ** 3 / const.lb)


def lbft3_2_kgm3(value):
    """
     converts density in lb/ft3(pounds/cubic feet) to kgm3(kilogrammes/cubic meter)
     :param value: density in lb/ft3(pounds/cubic feet)
     :return: density in kgm3(kilogrammes/cubic meter)
     """
    return value / (const.foot ** 3 / const.lb)


# compressibility


def compr_1pa_2_1psi(value):
    """
     converts compressibility in 1/pa to 1/psi
     :param value: compressibility in 1/pa
     :return: compressibility in 1/psi
     """
    return value * const.psi


def compr_1psi_2_1Pa(value):
    """
     converts compressibility in 1/psi to 1/pa
     :param value: compressibility in 1/psi
     :return: compressibility in 1/pa
     """
    return value / const.psi


def compr_1psi_2_1MPa(value):
    """
     converts compressibility in 1/psi to 1/pa
     :param value: compressibility in 1/psi
     :return: compressibility in 1/pa
     """
    return value / const.psi * const.mega

def compr_1pa_2_1bar(value):
    """
     converts compressibility in 1/pa to 1/bar
     :param value: compressibility in 1/pa
     :return: compressibility in 1/bar
     """
    return value * const.bar


def compr_1bar_2_1pa(value):
    """
     converts compressibility in 1/bar to 1/pa
     :param value: compressibility in 1/bar
     :return: compressibility in 1/pa
     """
    return value / const.bar


def compr_1mpa_2_1bar(value):
    """
     converts compressibility in 1/mpa to 1/bar
     :param value: compressibility in 1/mpa
     :return: compressibility in 1/bar
     """
    return value * const.bar / const.mega


def compr_1bar_2_1mpa(value):
    """
     converts compressibility in 1/bar to 1/mpa
     :param value: compressibility in 1/bar
     :return: compressibility in 1/mpa
     """
    return value * const.mega / const.bar


# heat capacity


def btulbmF_2_kJkgK(value):
    """
    converts heat capacity from FIELD to SI
    :param value: heat capacity in Btu/lbm F
    :return: heat capacity in kJ/kgK equal to kJ/kgC
    """
    return value * 4.186


# surface tension


def dyncm_2_Nm(value):
    """
    converts surface tension
    :param value: surface tension in dyn / cm
    :return: surface tension in N / m
    """
    return value / 1000


def Nm_2_dyncm(value):
    """
    converts surface tension
    :param value: surface tension in dyn / cm
    :return: surface tension in N / m
    """
    return value * 1000

# volumetric rates

def m3day_2_m3sec(value):
    """
    comverts volumetric flow rates from m3 per day to m3 per second
    :param value: flow volume in m3  / day
    :return: flow volume in m3  / sec
    """
    return value / 86400


def m3sec_2_m3day(value):
    """
    comverts volumetric flow rates from m3 per second to m3 per day
    :param value: flow volume in m3  / day
    :return: flow volume in m3  / sec
    """
    return value * 86400


def m3day_2_bblday(value):
    """
    comverts volumetric flow rates from m3 per day to barrel per day
    :param value: flow volume in m3  / day
    :return: flow volume in barrel  / day
    """
    return value / const.barrel


def bblday_2_m3day(value):
    """
    comverts volumetric flow rates from  barrel per day to m3 per day
    :param value: flow volume in m3  / day
    :return: flow volume in barrel  / day
    """
    return value * const.barrel

    
# viscosity 


def cP_2_Pasec(value):
    """
    converts viscosity
    :param value: viscosity in cP
    :return: viscosity in pa * sec
    """
    return value / 1000

# angles

def grad_2_rad(value):
    """
    converts angle from degrees to radians
    :param value: angle in grad
    :return: angle in rad
    """
    return value * pi / 180




"""
реализация модели нелетучей нефти
"""

import numpy as np
import neftpy.uconvert as uc
import neftpy.upvt as upvt
import scipy.optimize as opt

class BlackOilStanding:

    def __init__(self, 
                gamma_oil=0.86, 
                gamma_gas=0.6, 
                gamma_wat=1.0, 
                rsb_m3m3=100.0,
                tb_C=80, 
                y_h2s=0, 
                y_co2=0,
                y_n2=0  ):
        """
        Создает флюид с заданными базовыми свойствами

        калибровочные параметры при необходимости надо задавать отдельно

        :param gamma_oil: specific gravity of oil
        :param gamma_gas: specific gravity of gas (by air), dimensionless
        :param gamma_wat: specific gravity of water
        :param rsb_m3m3: solution gas ratio at bubble point
        :param tb_C: reservoir temperature 
        :param y_h2s: mole fraction of the hydrogen sulfide
        :param y_co2: mole fraction of the carbon dioxide
        :param y_n2: mole fraction of the nitrogen
        """
        self.gamma_gas = gamma_gas
        self.gamma_oil = gamma_oil
        self.gamma_wat = gamma_wat
        self.rsb_m3m3 = rsb_m3m3
        self.tb_C = tb_C  
        self.y_h2s = y_h2s
        self.y_co2 = y_co2
        self.y_n2 = y_n2
        # термобарические условия для которых рассчитаны свойства
        self.p_bar = uc.pressure_sc_bar                 # thermobaric conditions for all parameters
        self.t_c = uc.temperature_sc_C                  # can be set up by calc method

        self.__p_calibrated_MPaa =  uc.bar_2_MPa(self.p_bar)

        # расчетные параметры доступны только для чтения через свойства

        self.pb_bar = 0.0
        self.rs_m3m3 = 0.0
        self.rho_oil_kgm3 = 0.0
        self.b_oilb_m3m3 = 0.0
        self.b_oil_m3m3 = 1.0
        self.compr_oil_1bar = 0.0

        self.mu_dead_oil_cP = 0.0
        self.mu_oilb_cP = 0.0
        self.mu_oil_cP = 0.0  
    
        self.mu_gas_cP = 0.0
        self.mu_wat_cP = 0.0
        self.rho_gas_kgm3 = 0.0
        self.rho_wat_kgm3 = 0.0
        self.rsw_m3m3 = 0.0   # TODO проверить - равен GWR?
        self.b_gas_m3m3 = 0.0
        self.b_wat_m3m3 = 0.0
        self.z = 0.0
        self.compr_gas_1bar = 0.0
        self.compr_wat_1bar = 0.0
        self.heatcap_oil_jkgc = 0.0
        self.heatcap_gas_jkgc = 0.0
        self.heatcap_wat_jkgc = 0.0
        self.sigma_oil_gas_Nm = 0.0
        self.sigma_wat_gas_Nm = 0.0
        self.thermal_conduct_oil_wmk = 0.0
        self.thermal_conduct_gas_wmk = 0.0
        self.thermal_conduct_wat_wmk = 0.0
        self.thermal_expansion_wat_1c = 0.0
        
        self.activate_rus_cor = 0

        # калибровочные параметры для внутренних расчетов
        self.p_cal_mult = 1
        self.b_oil_cal_mult = 1
        self.mu_cal_mult = 1

        self.pb_calibr_bar = 0.0
        self.tb_calibr_C = 0.0
        self.b_oilb_calibr_m3m3 = 0.0
        self.mu_oilb_calibr_cP = 0.0

    def set_calibration_param(
                self,
                pb_calibr_bar=-1., 
                tb_calibr_C=80, 
                b_oilb_calibr_m3m3=0, 
                mu_oilb_calibr_cP=0.5
                ):
        # установка калибровочных параметров  
        # как отдельный метод, чтобы в явном виде обратить внимание пользователя на 
        # механизм калибровки
        self.pb_calibr_bar = pb_calibr_bar
        self.tb_calibr_C = tb_calibr_C
        self.b_oilb_calibr_m3m3 = b_oilb_calibr_m3m3
        self.mu_oilb_calibr_cP = mu_oilb_calibr_cP
        pass


    def __calc_pb_MPaa(self):
        self.pb_MPaa = upvt.unf_pb_Standing_MPaa(self.rsb_m3m3, self.gamma_oil, self.gamma_gas, self.t_K)
        self.pb_bar = uc.MPa_2_bar(self.pb_MPaa)

        # найдем калибровочный коэффициент для давления насыщения
        self.p_cal_mult = np.where(self.pb_calibr_bar > 0, 
                                   self.pb_bar / self.pb_calibr_bar, 
                                   1.)  
        self.__p_calibrated_MPaa = uc.bar_2_MPa(self.p_bar * self.p_cal_mult)

    def __calc_rs_m3m3(self):
        self.rs_m3m3 = upvt.unf_rs_Standing_m3m3(self.__p_calibrated_MPaa, self.pb_MPaa, self.rsb_m3m3, self.gamma_oil, self.gamma_gas, self.t_K)

    #def _calc_compressibility_oil_1Mpa(self):
    #    return upvt.unf_compressibility_oil_VB_1Mpa(self._rs_m3m3, self.t_K, self.gamma_oil, self.pcal_MPaa, self.gamma_gas)

    #def _calc_b_oilb_m3m3(self):
    #    return upvt.unf_bo_saturated_Standing_m3m3(self.rsb_m3m3, self.gamma_gas, self.gamma_oil, self.t_K)

    def __calc_b_rho_compressibility_oil(self):
        # поскольку объемный коэффициент, плотность и сжимаемость при давлениях выше pb 
        # тесно связаны между собой - считаем их вместе
        co_1MPa = upvt.unf_compressibility_oil_VB_1Mpa(self.rs_m3m3, self.t_K, self.gamma_oil, self.__p_calibrated_MPaa, self.gamma_gas)
        self.compr_oil_1bar = uc.compr_1mpa_2_1bar(co_1MPa)

        # оценим значение объемного коэффициента
        # оценим значение объемного коэффициента при давлении насыщения
        self.b_oilb_m3m3 = upvt.unf_bo_saturated_Standing_m3m3(self.rsb_m3m3, self.gamma_gas, self.gamma_oil, self.t_K)
        self.__b_oil_m3m3 = np.where(self.__p_calibrated_MPaa > self.pb_MPaa, 
                                 upvt.unf_bo_above_pb_m3m3(self.b_oilb_m3m3, co_1MPa, self.pb_MPaa, self.__p_calibrated_MPaa),
                                 upvt.unf_bo_saturated_Standing_m3m3(self.rs_m3m3, self.gamma_gas, self.gamma_oil, self.t_K)
                                )
        
        # проверим необходимость калибровки значения объемного коэффициента
        self.b_oil_cal_mult = np.where(self.b_oilb_calibr_m3m3 > 0, (self.b_oilb_calibr_m3m3 - 1) / (self.b_oilb_m3m3 - 1), 1.)
        self.b_oil_m3m3 = 1 + self.b_oil_cal_mult * (self.__b_oil_m3m3 - 1)
        # плотность нефти    
        self.rho_oil_kgm3 = upvt.unf_density_oil_Standing(self.__p_calibrated_MPaa , self.pb_MPaa, co_1MPa, self.rs_m3m3, self.b_oil_m3m3,
                                                          self.gamma_gas, self.gamma_oil) 
        

    def __calc_mu_oil_cP(self):
        # оценим значение вязкости
        self.mu_dead_oil_cP = upvt.unf_deadoilviscosity_Beggs_cP(self.gamma_oil, self.t_K)
        self.mu_oilb_cP = upvt.unf_saturatedoilviscosity_Beggs_cP(self.mu_dead_oil_cP, self.rsb_m3m3)
        self.mu_oil_cP = upvt.unf_oil_viscosity_Beggs_VB_cP(self.mu_dead_oil_cP, self.rs_m3m3, self.__p_calibrated_MPaa , self.pb_MPaa)
        if self.mu_oilb_calibr_cP > 0:
            self.mu_cal_mult = self.mu_oilb_calibr_cP / self.mu_oilb_cP
            self.mu_oil_cP = self.mu_cal_mult * self.mu_oil_cP
        

    def __calc_termodynamic_oil_props(self):
        # определим термодинамические свойства нефти
        self._heatcap_oil_jkgc = upvt.unf_heat_capacity_oil_Gambill_JkgC(self.gamma_oil, self.t_c)
        self._thermal_conduct_oil_wmk = upvt.unf_thermal_conductivity_oil_Cragoe_WmK(self.gamma_oil, self.t_c)

    def calc(self, p_bar, t_C):
        self.p_bar = p_bar
        self.t_C = t_C 
        # в расчете часто используются MPa и K - сконвертируем и сохраним соответствующие значения
        self.p_MPaa = uc.bar_2_MPa(p_bar)
        self.t_K = uc.C_2_K(self.t_C)

        # оценим свойства нефти 
        self.__calc_pb_MPaa()       # давление насыщения

        self.__calc_rs_m3m3()       # газосодержание 
        
        self.__calc_b_rho_compressibility_oil()   # объемный коэффициент нефти, плотность нефти, сжимаемость нефти
        
        self.__calc_mu_oil_cP()     # вязкость нефти

        self.__calc_termodynamic_oil_props()

        # свойства газа 

        # свойства воды

"""
        # определим термодинамические свойства нефти
        self._heatcap_oil_jkgc = self._calc_heat_capacity_oil_JkgC()
        self._thermal_conduct_oil_wmk = self._calc_thermal_conductivity_oil_WmK()


        # gas
        tpc_K = PVT.unf_pseudocritical_temperature_K(self.gamma_gas, self.y_h2s, self.y_co2, self.y_n2)
        ppc_MPa = PVT.unf_pseudocritical_pressure_MPa(self.gamma_gas, self.y_h2s, self.y_co2, self.y_n2)
        self._z = PVT.unf_zfactor_DAK(p_MPaa, t_K, ppc_MPa, tpc_K)
        self._mu_gas_cP = PVT.unf_gasviscosity_Lee_cP(t_K, p_MPaa, self._z, self.gamma_gas)
        self._bg_m3m3 = PVT.unf_gas_fvf_m3m3(t_K, p_MPaa, self._z)
        self._compr_gas_1bar = uc.compr_1mpa_2_1bar(PVT.unf_compressibility_gas_Mattar_1MPa(p_MPaa, t_K,
                                                                                            ppc_MPa, tpc_K))
        self._rho_gas_kgm3 = PVT.unf_gas_density_kgm3(t_K, p_MPaa, self.gamma_gas, self._z)

        # определим термодинамические свойства газа
        self._heatcap_gas_jkgc = PVT.unf_heat_capacity_gas_Mahmood_Moshfeghian_JkgC(p_MPaa, t_K, self.gamma_gas)
        self._thermal_conduct_gas_wmk = PVT.unf_thermal_conductivity_gas_methane_WmK(self.t_c)

        # water
        # TODO НУЖНО проверить GWR
        self._rho_wat_kgm3 = PVT.unf_density_brine_Spivey_kgm3(t_K, p_MPaa, self.s_ppm, self.par_wat)
        self._compr_wat_1bar = uc.compr_1mpa_2_1bar(PVT.unf_compressibility_brine_Spivey_1MPa(t_K, p_MPaa, self.s_ppm,
                                                                                              self._z, self.par_wat))
        self._bw_m3m3 = PVT.unf_fvf_brine_Spivey_m3m3(t_K, p_MPaa, self.s_ppm)
        self._mu_wat_cP = PVT.unf_viscosity_brine_MaoDuan_cP(t_K, p_MPaa, self.s_ppm)
        self._rsw_m3m3 = PVT.unf_gwr_brine_Spivey_m3m3(self.s_ppm, self._z)
        # определим термодинамические свойства воды
        self._heatcap_wat_jkgc = PVT.unf_heat_capacity_water_IAPWS_JkgC(self.t_c)
        self._thermal_conduct_wat_wmk = PVT.unf_thermal_conductivity_water_IAPWS_WmC(self.t_c)
        self._thermal_expansion_wat_1c = PVT.unf_thermal_expansion_coefficient_water_IAPWS_1C(self.t_c)

        # определим свойства системы
        self._sigma_oil_gas_Nm = PVT.unf_surface_tension_go_Baker_Swerdloff_Nm(t_K, self.gamma_oil, p_MPaa)
        self._sigma_wat_gas_Nm = PVT.unf_surface_tension_gw_Sutton_Nm(self.rho_wat_kgm3, self.rho_gas_kgm3, self.t_c)
        return 1
"""



from setuptools import setup, find_packages
from os.path import join, dirname
import neftpy

setup(
    name='neftpy',
    version=neftpy.__version__,
    description='Petroluem production engineering calculations package. Inspired by Unifloc VBA.',
    packages=find_packages(),
    long_description=open(join(dirname(__file__), 'README.md')).read(),
    url='https://github.com/unifloc/neftpy',
    author='Rinat Khabibullin',
    author_email='khabibullinra@gmail.com',
    license='BSD 3-Clause License',
    install_requires=[
          'numpy', 'scipy',
      ],
)


"""
PVT (pressure volume temperature) functions based on Black Oil PVT model for petroleum engineering calculations
old versions from vba (without vectorization)

Rinat Khabibullin
revision from 13/05/2022

Unifloc_VBA and unifloc_py refactoring
"""

import numpy as np
import neftpy.uconvert as uc
import scipy.optimize as opt

""" 
====================================================================================================
Корреляции расчета давления насыщения
====================================================================================================
"""

# простая реализация (без векторизации)
def _unf_pb_Standing_MPaa_(rsb_m3m3=100, gamma_oil=0.86, gamma_gas=0.6, t_K=350):
    """
        bubble point pressure calculation according to Standing (1947) correlation

    :param rsb_m3m3: solution ration at bubble point, must be given, m3/m3
    :param gamma_oil: specific oil density (by water)
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :return: bubble point pressure abs in MPa

    ref1 "A Pressure-Volume-Temperature Correlation for Mixtures of California Oil and Gases",
    M.B. Standing, Drill. & Prod. Prac., API, 1947.

    ref2  "Стандарт компании Юкос. Физические свойства нефти. Методы расчета." Афанасьев В.Ю., Хасанов М.М. и др. 2002 г
    """

    min_rsb = 1.8
    rsb_old = rsb_m3m3
    if rsb_m3m3 < min_rsb:
        rsb_m3m3 = min_rsb
    # мольная доля газа
    yg = 1.225 + 0.001648 * t_K - 1.769 / gamma_oil
    pb_MPaa = 0.5197 * (rsb_m3m3 / gamma_gas) ** 0.83 * 10 ** yg
    # для низких значений газосодержания зададим асимптотику Pb = 1 атма при Rsb = 0
    # для больших значений газосодержания не корректируем то, что дает корреляция
    if rsb_old < min_rsb:
        pb_MPaa = (pb_MPaa - 0.1013) * rsb_old / min_rsb + 0.1013  # 0.101325
    return pb_MPaa

# принудительно векторизованная версия 
unf_pb_Standing_MPaa = np.vectorize(_unf_pb_Standing_MPaa_)


# не векторизованная версия
def _unf_pb_Valko_MPaa_(rsb_m3m3=100, gamma_oil=0.86, gamma_gas=0.6, t_K=350):
    """
        bubble point pressure calculation according to Valko McCain (2002) correlation

    :param rsb_m3m3: solution ration at bubble point, must be given, m3/m3
    :param gamma_oil: specific oil density (by water)
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :return: bubble point pressure abs in MPa

    ref SPE  "Reservoir oil bubblepoint pressures revisited; solution gas–oil ratios and surface gas specific gravities"
    W. D. McCain Jr.,P.P. Valko,
    """

    min_rsb = 1.8
    max_rsb = 800
    rsb_old = rsb_m3m3
    if rsb_m3m3 < min_rsb:
        rsb_m3m3 = min_rsb
    if rsb_m3m3 > max_rsb:
        rsb_m3m3 = max_rsb

    z1 = -4.81413889469569 + 0.748104504934282 * np.log(rsb_m3m3) \
        + 0.174372295950536 * np.log(rsb_m3m3) ** 2 - 0.0206 * np.log(rsb_m3m3) ** 3
    z2 = 25.537681965 - 57.519938195 / gamma_oil + 46.327882495 / gamma_oil**2 \
         - 13.485786265 / gamma_oil ** 3
    z3 = 4.51 - 10.84 * gamma_gas + 8.39 * gamma_gas ** 2 - 2.34 * gamma_gas ** 3
    z4 = -7.2254661 + 0.043155 * t_K - 8.5548e-5 * t_K ** 2 + 6.00696e-8 * t_K ** 3
    z = z1 + z2 + z3 + z4

    pb_MPaa = 12.1582266504102 * np.exp(0.0075 * z**2 + 0.713 * z)

    """
    для низких значений газосодержания зададим асимптотику Pb = 1 атма при Rsb = 0
    корреляция Valko получена с использованием непараметрической регресии GRACE метод (SPE 35412)
    особенность подхода - за пределеми интервала адаптации ассимптотики не соблюдаются
    поэтому их устанавливаем вручную
    для больших значений газосодержания продолжим линейный тренд корреляции
    """
    if rsb_old < min_rsb:
        pb_MPaa = (pb_MPaa - 0.1013) * rsb_old / min_rsb + 0.1013
    if rsb_old > max_rsb:
        pb_MPaa = (pb_MPaa - 0.1013) * rsb_old / max_rsb + 0.1013

    return pb_MPaa

# наивная векторизация
unf_pb_Valko_MPaa = np.vectorize(_unf_pb_Valko_MPaa_)


""" 
====================================================================================================
Корреляции расчета газосодержания (Gas Solution Ratio)
====================================================================================================
"""


# простой расчет без векторизации
def _unf_rs_Standing_m3m3_(p_MPaa=1, pb_MPaa=10, rsb_m3m3=0, gamma_oil=0.86, gamma_gas=0.6, t_K=350):
    """
        Gas-oil ratio calculation inverse of Standing (1947) correlation for bubble point pressure

    :param p_MPaa: pressure, MPa
    :param pb_MPaa: buble point pressure, MPa
    :param rsb_m3m3: gas-oil ratio at the bubble point pressure, m3/m3
    :param gamma_oil: specific oil density (by water)
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :return: gas-oil ratio in m3/m3

    ref1 "A Pressure-Volume-Temperature Correlation for Mixtures of California Oil and Gases",
    M.B. Standing, Drill. & Prod. Prac., API, 1947.

    ref2  "Стандарт компании Юкос. Физические свойства нефти. Методы расчета." Афанасьев В.Ю., Хасанов М.М. и др. 2002 г
    может считать в случае если нет давления насыщения и газосодержания при давлении насыщения, корреляция не точная
    """

    if pb_MPaa == 0 or rsb_m3m3 == 0:
        # мольная доля газа
        yg = 1.225 + 0.001648 * t_K - 1.769 / gamma_oil
        rs_m3m3 = gamma_gas * (1.92 * p_MPaa / 10 ** yg) ** 1.204
    elif p_MPaa < pb_MPaa:
        rs_m3m3 = rsb_m3m3 * (p_MPaa / pb_MPaa) ** 1.204
    else:
        rs_m3m3 = rsb_m3m3
    return rs_m3m3

# наивная векторизация
unf_rs_Standing_m3m3 = np.vectorize(_unf_rs_Standing_m3m3_)


def _unf_rs_Velarde_m3m3_(p_MPaa=1, pb_MPaa=10, rsb_m3m3=100., gamma_oil=0.86, gamma_gas=0.6, t_K=350):
    """
        Solution Gas-oil ratio calculation according to Velarde McCain (1999) correlation

    :param p_MPaa: pressure, MPa
    :param pb_MPaa: buble point pressure, MPa
    :param rsb_m3m3: gas-oil ratio at the bubble point pressure, m3/m3
    :param gamma_oil: specific oil density (by water)
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :return: gas-oil ratio in m3/m3

        ref1 "Correlation of Black Oil Properties at Pressures Below Bubblepoint Pressure—A New Approach",
    J. VELARDE, T.A. BLASINGAME Texas A&M University, W.D. MCCAIN, JR. S.A. Holditch & Associates, Inc 1999

    """

    api = uc.gamma_oil_2_api(gamma_oil)
    t_F = uc.K_2_F(t_K)
    pb_psig = uc.MPa_2_psi(pb_MPaa) - 14.7

    if pb_psig > 0:
        pr = uc.MPa_2_psig(p_MPaa)  / pb_psig
    else:
        pr = 0

    if pr <= 0:
        rs_m3m3 = 0.0
    elif pr < 1:
        A = np.array([9.73 * 10 ** (-7), 1.672608, 0.929870, 0.247235, 1.056052])
        B = np.array([0.022339, -1.004750, 0.337711, 0.132795, 0.302065])
        C = np.array([0.725167, -1.485480, -0.164741, -0.091330, 0.047094])
        
        a1 = A[0] * gamma_gas ** A[1] * api ** A[2] * t_F ** A[3] * pb_psig ** A[4]
        a2 = B[0] * gamma_gas ** B[1] * api ** B[2] * t_F ** B[3] * pb_psig ** B[4]
        a3 = C[0] * gamma_gas ** C[1] * api ** C[2] * t_F ** C[3] * pb_psig ** C[4]
        pr = uc.MPa_2_psig(p_MPaa)  / pb_psig
        rsr = a1 * pr ** a2 + (1 - a1) * pr ** a3
        rs_m3m3 = rsr * rsb_m3m3
    else:
        rs_m3m3 = rsb_m3m3
    return rs_m3m3

# наивная векторизация
unf_rs_Velarde_m3m3 = np.vectorize(_unf_rs_Velarde_m3m3_)


def _unf_rsb_Mccain_m3m3_(rsp_m3m3=10, gamma_oil=0.86, psp_MPaa=0.0, tsp_K=0.0):
    """
        Solution Gas-oil ratio at bubble point pressure calculation according to McCain (2002) correlation
    taking into account the gas losses at separator and stock tank

    :param rsp_m3m3: separator producing gas-oil ratio, m3m3
    :param gamma_oil: specific oil density(by water)
    :param psp_MPaa: pressure in separator, MPaa
    :param tsp_K: temperature in separator, K
    :return: solution gas-oil ratio at bubble point pressure, rsb in m3/m3

    часто условия в сепараторе неизвестны, может считать и без них по приблизительной формуле

    ref1 "Reservoir oil bubblepoint pressures revisited; solution gas–oil ratios and surface gas specific gravities",
    J. VELARDE, W.D. MCCAIN, 2002
    """

    rsp_scfstb = uc.m3m3_2_scfstb(rsp_m3m3)
    if psp_MPaa > 0 and tsp_K > 0:
        api = uc.gamma_oil_2_api(gamma_oil)
        psp_psia = uc.Pa_2_psi(psp_MPaa * 10 ** 6)
        tsp_F = uc.K_2_F(tsp_K)
        z1 = -8.005 + 2.7 * np.log(psp_psia) - 0.161 * np.log(psp_psia) ** 2
        z2 = 1.224 - 0.5 * np.log(tsp_F)
        z3 = -1.587 + 0.0441 * np.log(api) - 2.29 * 10 ** (-5) * np.log(api) ** 2
        z = z1 + z2 + z3
        rst_scfstb = np.exp(3.955 + 0.83 * z - 0.024 * z ** 2 + 0.075 * z ** 3)
        rsb = rsp_scfstb + rst_scfstb
    elif rsp_m3m3 >= 0:
        rsb = 1.1618 * rsp_scfstb
    else:
        rsb = 0
    rsb = uc.scfstb_2_m3m3(rsb)
    return rsb

# наивная векторизация
unf_rsb_Mccain_m3m3 = np.vectorize(_unf_rsb_Mccain_m3m3_)


""" 
====================================================================================================
Корреляции расчета объемного коэффициента нефти (Oil Formation Volume Factor FVF)
====================================================================================================
"""

def _unf_bo_above_pb_m3m3_(bob_m3m3=1.2, compr_o_1MPa=3e-3, pb_MPaa=10, p_MPaa=1):
    """
        Oil Formation Volume Factor according equation for pressure above bubble point pressure

    :param bob_m3m3: formation volume factor at bubble point pressure, m3m3
    :param compr_o_1MPa: weighted-average oil compressibility from bubblepoint pressure to a higher pressure of interest,1/MPa
    :param pb_MPaa: bubble point pressure, MPa
    :param p_MPaa: pressure, MPa
    :return: formation volume factor bo,m3m3

    ref1 book Mccain_w_d_spivey_j_p_lenn_c_p_petroleum_reservoir_fluid,third edition, 2011

    ! Actually, this correlation is belonged ro Vasquez & Beggs (1980). In some sources is
    noted that this is Standing correlation.

    ref2 Vazquez, M. and Beggs, H.D. 1980. Correlations for Fluid Physical Property Prediction.
    J Pet Technol 32 (6): 968-970. SPE-6719-PA

    """

    if p_MPaa <= pb_MPaa:
        bo = bob_m3m3
    else:
        bo = bob_m3m3 * np.exp(compr_o_1MPa * (pb_MPaa - p_MPaa))
    return bo

# наивная векторизация
unf_bo_above_pb_m3m3 = np.vectorize(_unf_bo_above_pb_m3m3_)



def unf_bo_below_pb_m3m3(rho_oil_st_kgm3=820, rs_m3m3=100, rho_oil_insitu_kgm3=700, gamma_gas=0.8):
    """
        Oil Formation Volume Factor according McCain correlation for pressure below bubble point pressure

    :param rho_oil_st_kgm3: density of stock-tank oil, kgm3
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param rho_oil_insitu_kgm3: Oil density at reservoir conditions, kgm3
    :param gamma_gas: specific gas  density(by air)
    :return: formation volume factor bo, m3m3

    ref1 book Mccain_w_d_spivey_j_p_lenn_c_p_petroleum_reservoir_fluid,third edition, 2011
    """

    rho_oil_st_lbft3 = uc.kgm3_2_lbft3(rho_oil_st_kgm3)
    rho_oil_insitu_lbft3 = uc.kgm3_2_lbft3(rho_oil_insitu_kgm3)
    rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
    bo = (rho_oil_st_lbft3 + 0.01357 * rs_scfstb * gamma_gas) / rho_oil_insitu_lbft3
    return bo


def unf_bo_saturated_Standing_m3m3(rs_m3m3=100, gamma_gas=0.8, gamma_oil=0.86, t_K=300):
    """
        Oil Formation Volume Factor according Standing equation at bubble point pressure

    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param gamma_gas: specific gas density (by air)
    :param gamma_oil: specific oil density (by water)
    :param t_K: temperature, K
    :return: formation volume factor at bubble point pressure bo,m3m3

    ref1 Volumetric and phase behavior of oil field hydrocarbon systems / M.B. Standing Standing, M. B. 1981
    """

    rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
    t_F = uc.K_2_F(t_K)
    bo = 0.972 + 1.47e-4 * (rs_scfstb * (gamma_gas / gamma_oil) ** 0.5 + 1.25 * t_F) ** 1.175
    return bo


""" 
====================================================================================================
Корреляции расчета плотности нефти
====================================================================================================
"""

def _unf_density_oil_Mccain_(p_MPaa=1, pb_MPaa=10, co_1MPa=3e-3, rs_m3m3=10, gamma_gas=0.86, t_K=300, gamma_oil=0.86, gamma_gassp = 0):
    """
        Oil density according Standing, M.B., 1977; Witte, T.W., Jr., 1987; and McCain, W.D., Jr. and Hill, N.C.,
    1995 correlation.

    :param p_MPaa: pressure, MPaa
    :param pb_MPaa: bubble point pressure, MPaa
    :param co_1MPa: coefficient of isothermal compressibility, 1/MPa
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :param gamma_oil: specific oil density (by water)
    :param gamma_gassp: specific gas density in separator(by air)
    :return: oil density,kg/m3

    ref1 book Mccain_w_d_spivey_j_p_lenn_c_p_petroleum_reservoir_fluid,third edition, 2011
    """
    rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
    t_F = uc.K_2_F(t_K)
    p_psia = uc.MPa_2_psi(p_MPaa)

    if gamma_gassp == 0:  # если нет данных о плотности газа в сепараторе
        gamma_gassp = gamma_gas
    def ro_po_equation(ro_po, vars):
        gamma_gassp, gamma_gas, gamma_oil, rs_scfstb = vars
        ro_a = -49.8930 + 85.0149 * gamma_gassp - 3.70373 * gamma_gassp * ro_po +\
            0.0479818 * gamma_gassp * ro_po ** 2 + 2.98914 * ro_po - 0.0356888 * ro_po ** 2
        return (rs_scfstb * gamma_gas + 4600 * gamma_oil) / (73.71 + rs_scfstb * gamma_gas / ro_a) -ro_po
    
    ro_po = 52.8 - 0.01 * rs_scfstb  # первое приближение
    ro_po = opt.fsolve(ro_po_equation, ro_po, [gamma_gassp, gamma_gas, gamma_oil, rs_scfstb])[0]
    
    if p_MPaa < pb_MPaa:
        p = p_psia 
    else:
        pb_psia = uc.MPa_2_psi(pb_MPaa)
        p = pb_psia

    dro_p = ((0.167 + 16.181 * 10 ** (-0.0425 * ro_po)) * (p / 1000) - 0.01 * 
             (0.299 + 263 * 10 ** (-0.0603 * ro_po)) * (p / 1000) ** 2)
    ro_bs = ro_po + dro_p
    dro_t = ((0.00302 + 1.505 * ro_bs ** (-0.951)) * (t_F - 60) ** 0.938 - 
             (0.0216 - 0.0233 * 10 ** (-0.0161 * ro_bs)) * (t_F - 60) ** 0.475)
    ro_or = ro_bs - dro_t

    if p_MPaa >= pb_MPaa:
        co_1psi = uc.compr_1pa_2_1psi(co_1MPa / 10 ** 6)
        ro_or = ro_or * np.exp(co_1psi * (p_psia - pb_psia))
    ro_or = uc.lbft3_2_kgm3(ro_or)
    return ro_or

# векторная версия для единообразия
unf_density_oil_Mccain = np.vectorize(_unf_density_oil_Mccain_)

# старая версия с решением уравнения из VBA
# оставлено для справки
def __unf_density_oil_Mccain__(p_MPaa=1, pb_MPaa=10, co_1MPa=3e-3, rs_m3m3=10, gamma_gas=0.8, t_K=300, gamma_oil=0.86, gamma_gassp = 0):
    """
        Oil density according Standing, M.B., 1977; Witte, T.W., Jr., 1987; and McCain, W.D., Jr. and Hill, N.C.,
    1995 correlation.

    :param p_MPaa: pressure, MPaa
    :param pb_MPaa: bubble point pressure, MPaa
    :param co_1MPa: coefficient of isothermal compressibility, 1/MPa
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param gamma_gas: specific gas density (by air)
    :param t_K: temperature, K
    :param gamma_oil: specific oil density (by water)
    :param gamma_gassp: specific gas density in separator(by air)
    :return: oil density,kg/m3

    ref1 book Mccain_w_d_spivey_j_p_lenn_c_p_petroleum_reservoir_fluid,third edition, 2011
    """

    rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
    ro_po = 52.8 - 0.01 * rs_scfstb  # первое приближение
    if gamma_gassp == 0:  # если нет данных о плотности газа в сепараторе
        gamma_gassp = gamma_gas
    epsilon = 0.000001
    maxiter = 100  # максимальное число иттераций
    counter = 0
    ro_po_current = ro_po
    ro_po_previous = 0
    # TODO в идеале нужно также как для z-фактора сделать через scipy а не через цикл
    while abs(ro_po_current - ro_po_previous) > epsilon and counter < maxiter:
        ro_po_previous = ro_po_current
        ro_a = -49.8930 + 85.0149 * gamma_gassp - 3.70373 * gamma_gassp * ro_po +\
            0.0479818 * gamma_gassp * ro_po ** 2 + 2.98914 * ro_po - 0.0356888 * ro_po ** 2
        ro_po = (rs_scfstb * gamma_gas + 4600 * gamma_oil) / (73.71 + rs_scfstb * gamma_gas / ro_a)
        ro_po_current = ro_po
        counter = counter + 1
    p_psia = uc.Pa_2_psi(p_MPaa * 10 ** 6)
    t_F = uc.K_2_F(t_K)
    if p_MPaa < pb_MPaa:
        dro_p = (0.167 + 16.181 * 10 ** (-0.0425 * ro_po)) * (p_psia / 1000) - 0.01 * (
                    0.299 + 263 * 10 ** (-0.0603 * ro_po)) * (p_psia / 1000) ** 2
        ro_bs = ro_po + dro_p
        dro_t = (0.00302 + 1.505 * ro_bs ** (-0.951)) * (t_F - 60) ** 0.938 - (0.0216 - 0.0233 * 10 ** (-0.0161 * ro_bs)
                                                                               ) * (t_F - 60) ** 0.475
        ro_or = ro_bs - dro_t
    else:
        pb_psia = uc.Pa_2_psi(pb_MPaa * 10 ** 6)
        dro_p = (0.167 + 16.181 * 10 ** (-0.0425 * ro_po)) * (pb_psia / 1000) - 0.01 * (
                    0.299 + 263 * 10 ** (-0.0603 * ro_po)) * (pb_psia / 1000) ** 2
        ro_bs = ro_po + dro_p
        dro_t = (0.00302 + 1.505 * ro_bs ** (-0.951)) * (t_F - 60) ** 0.938 - (
                    0.0216 - 0.0233 * 10 ** (-0.0161 * ro_bs)) * (t_F - 60) ** 0.475
        ro_orb = ro_bs - dro_t
        co_1psi = uc.compr_1pa_2_1psi(co_1MPa / 10 ** 6)
        ro_or = ro_orb * np.exp(co_1psi * (p_psia - pb_psia))
    ro_or = uc.lbft3_2_kgm3(ro_or)
    return ro_or

    
def unf_density_oil_Standing(p_MPaa=1, pb_MPaa=10, co_1MPa=3e-3, rs_m3m3=10, bo_m3m3=1.1, gamma_gas=0.8, gamma_oil=0.86):
    """
        Oil density according Standing correlation.

    :param p_MPaa: pressure, MPaa
    :param pb_MPaa: bubble point pressure, MPaa
    :param co_1MPa: coefficient of isothermal compressibility, 1/MPa
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param bo_m3m3: oil formation volume factor, m3m3
    :param gamma_gas: specific gas density (by air)
    :param gamma_oil: specific oil density (by water)
    :return: oil density,kg/m3

    ref1 book Brill 2006, Production Optimization Using Nodal Analysis
    """
    po = (1000 * gamma_oil + 1.224 * gamma_gas * rs_m3m3) / bo_m3m3

    # TODO возможно это надо оставить, при давлении выше P нас, уже есть в fvf_oil степень с co
        
    return np.where(p_MPaa > pb_MPaa, po * np.exp(co_1MPa * (p_MPaa - pb_MPaa)), po)


""" 
====================================================================================================
Корреляции расчета сжимаемости нефти
====================================================================================================
"""
def unf_compressibility_saturated_oil_McCain_1Mpa(p_MPa=1, pb_MPa=10, t_K=300, gamma_oil=0.86, rsb_m3m3=100):
    """
        Oil compressibility below bubble point (saturated oil)

    :param p_mpa:
    :param pb_mpa:
    :param t_k:
    :param gamma_oil:
    :param rsb_m3m3:
    :return:

    ref1 https://www.researchgate.net/publication/
    254529353_The_Oil_Compressibility_Below_Bubble_Point_Pressure_Revisited_-_Formulations_and_Estimations

    ref2 https://www.onepetro.org/download/journal-paper/SPE-15664-PA?id=journal-paper%2FSPE-15664-PA
    """
    rsb_scfstb = uc.m3m3_2_scfstb(rsb_m3m3)
    t_F = uc.K_2_F(t_K)
    api = uc.gamma_oil_2_api(gamma_oil)
    p_psia = uc.MPa_2_psi(p_MPa)
    pb_psia = uc.MPa_2_psi(pb_MPa)
    co_1psi = np.exp(-7.573 - 1.450 * np.log(p_psia) - 0.383 * np.log(pb_psia) + 1.402 * np.log(t_F) +
                     0.256 * np.log(api) + 0.449 * np.log(rsb_scfstb))
    co_1MPa = uc.compr_1psi_2_1MPa(co_1psi)# * 10 ** 5 # TODO надо разобраться и исправить
    return co_1MPa


def unf_compressibility_oil_VB_1Mpa(rs_m3m3, t_K, gamma_oil, p_MPaa, gamma_gas=0.6): # TODO above bubble point!!
    """
        Oil compressibility according to Vasquez & Beggs (1980) correlation

    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param t_K: temperature, K
    :param gamma_oil: specific oil density (by water)
    :param p_MPaa: pressure, MPaa
    :param gamma_gas: specific gas density (by air)
    :return: coefficient of isothermal compressibility co_1MPa, 1/MPa

    ref1 Vazquez, M. and Beggs, H.D. 1980. Correlations for Fluid Physical Property Prediction.
    J Pet Technol 32 (6): 968-970. SPE-6719-PA

    """

    if p_MPaa > 0:
        rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
        t_F = uc.K_2_F(t_K)
        api = uc.gamma_oil_2_api(gamma_oil)
        p_psia = uc.MPa_2_psi(p_MPaa)
        co_1MPa = uc.compr_1psi_2_1MPa( (-1433 + 5 * rs_scfstb + 17.2 * t_F - 1180 * gamma_gas + 12.61 * api) / (1e5 * p_psia))
    else:
        co_1MPa = 0
    return co_1MPa

def unf_compessibility_oil_VB_1MPa_vba(rsb_m3m3, t_k, gamma_oil, gamma_gas):
    co_1atm = (28.1 * rsb_m3m3 + 30.6 * t_k - 1180 * gamma_gas + 1784 / gamma_oil - 10910)
    return 1/uc.atm_2_bar(1/co_1atm)/10  #*))) **когда не дружишь с математикой ***когда слишком много времени потратил на uniflocpy

""" 
====================================================================================================
Корреляции расчета плотности газа на поверхности
====================================================================================================
"""
def unf_gamma_gas_Mccain(rsp_m3m3, rst_m3m3, gamma_gassp=0.8, gamma_oil=0.86, psp_MPaa=0, tsp_K=0):
    """
        Correlation for weighted-average specific gravities of surface gases

    :param rsp_m3m3: separator producing gas-oil ratio, m3m3
    :param rst_m3m3: stock-tank producing gas-oil ratio, m3m3
    :param gamma_gassp:  separator gas specific gravity
    :param gamma_oil: specific oil density(by water)
    :param psp_MPaa: pressure in separator, MPaa
    :param tsp_K: temperature in separator, K
    :return: weighted-average specific gravities of surface gases

    часто условия в сепараторе неизвестны, может считать и без них по приблизительной формуле

    ref1 "Reservoir oil bubblepoint pressures revisited; solution gas–oil ratios and surface gas specific gravities",
    J. VELARDE, W.D. MCCAIN, 2002

    """

    api = uc.gamma_oil_2_api(gamma_oil)
    psp_psia = uc.MPa_2_psi(psp_MPaa)
    tsp_F = uc.K_2_F(tsp_K)
    rsp_scfstb = uc.m3m3_2_scfstb(rsp_m3m3)
    rst_scfstb = uc.m3m3_2_scfstb(rst_m3m3)

    nplogpsp_psia = np.log(psp_psia) 
    nplogrsp_scfstb = np.log(rsp_scfstb)

    z1 = -17.275 + 7.9597 * nplogpsp_psia - 1.1013 * nplogpsp_psia ** 2 + 2.7735e-2 \
        * nplogpsp_psia ** 3 + 3.2287e-3 * nplogpsp_psia ** 4
    z2 = -0.3354 - 0.3346 * nplogrsp_scfstb + 0.1956 * nplogrsp_scfstb ** 2 - 3.4374e-2 \
        * nplogrsp_scfstb ** 3 + 2.08e-3 * nplogrsp_scfstb ** 4
    z3 = 3.705 - 0.4273 * api + 1.818e-2 * api ** 2 - 3.459e-4 \
        * api ** 3 + 2.505e-6 * api ** 4
    z4 = -155.52 + 629.61 * gamma_gassp - 957.38 * gamma_gassp ** 2 + 647.57 \
        * gamma_gassp ** 3 - 163.26 * gamma_gassp ** 4
    z5 = 2.085 - 7.097e-2 * tsp_F + 9.859e-4 * tsp_F ** 2 \
        - 6.312e-6 * tsp_F ** 3 + 1.4e-8 * tsp_F ** 4
    z = z1 + z2 + z3 + z4 + z5
    # Stock-tank gas specific gravity
    gamma_gasst = 1.219 + 0.198 * z + 0.0845 * z ** 2 + 0.03 * z ** 3 + 0.003 * z ** 4
      
    return np.where(psp_MPaa * rsp_m3m3 * rst_m3m3 * gamma_gassp > 0,
                    (gamma_gassp * rsp_scfstb + gamma_gasst * rst_scfstb) / (rsp_scfstb + rst_scfstb),
                    np.where(gamma_gassp >= 0,
                            1.066 * gamma_gassp,
                            0
                            )
                    )

""" 
====================================================================================================
Корреляции расчета вязкости нефти
====================================================================================================
"""


def unf_deadoilviscosity_Beggs_cP(gamma_oil, t_K):
    """
        Correlation for dead oil viscosity

    :param gamma_oil: specific oil density (by water)
    :param t_K: temperature, K
    :return: dead oil viscosity,cP

    ref1 Beggs, H.D. and Robinson, J.R. “Estimating the Viscosity of Crude Oil Systems.”
    Journal of Petroleum Technology. Vol. 27, No. 9 (1975)

    """
    api = uc.gamma_oil_2_api(gamma_oil)
    t_F = uc.K_2_F(t_K)
    c = 10 ** (3.0324 - 0.02023 * api) * t_F ** (-1.163)  
    return 10 ** c - 1

def unf_saturatedoilviscosity_Beggs_cP(deadoilviscosity_cP, rs_m3m3):
    """
        Correlation for oil viscosity for pressure below bubble point (for pb!!!)

    :param deadoilviscosity_cP: dead oil viscosity,cP
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :return: oil viscosity,cP

    ref1 Beggs, H.D. and Robinson, J.R. “Estimating the Viscosity of Crude Oil Systems.”
    Journal of Petroleum Technology. Vol. 27, No. 9 (1975)

    """
    rs_scfstb = uc.m3m3_2_scfstb(rs_m3m3)
    a = 10.715 * (rs_scfstb + 100) ** (-0.515)
    b = 5.44 * (rs_scfstb + 150) ** (-0.338)
    viscosity_cP = a * deadoilviscosity_cP ** b
    return viscosity_cP

def unf_undersaturatedoilviscosity_VB_cP(p_MPaa, pb_MPaa, bubblepointviscosity_cP):
    """
        Viscosity correlation for pressure above bubble point

    :param p_MPaa: pressure, MPaa
    :param pb_MPaa: bubble point pressure, MPaa
    :param bubblepointviscosity_cP: oil viscosity at bubble point pressure, cP
    :return: oil viscosity,cP

    ref2 Vazquez, M. and Beggs, H.D. 1980. Correlations for Fluid Physical Property Prediction.
    J Pet Technol 32 (6): 968-970. SPE-6719-PA

    """
    p_psia = uc.MPa_2_psi(p_MPaa)
    pb_psia = uc.MPa_2_psi(pb_MPaa)
    m = 2.6 * p_psia ** 1.187 * np.exp(-11.513 - 8.98e-5 * p_psia)
    viscosity_cP = bubblepointviscosity_cP * (p_psia / pb_psia) ** m
    return viscosity_cP

def unf_oil_viscosity_Beggs_VB_cP(deadoilviscosity_cP, rs_m3m3, p_MPaa, pb_MPaa):
    """
        Function for calculating the viscosity at any pressure

    :param deadoilviscosity_cP: dead oil viscosity,cP
    :param rs_m3m3: solution gas-oil ratio, m3m3
    :param p_MPaa: pressure, MPaa
    :param pb_MPaa: bubble point pressure, MPaa
    :return: oil viscosity,cP
    """
    saturatedviscosity_cP = unf_saturatedoilviscosity_Beggs_cP(deadoilviscosity_cP, rs_m3m3)
    if p_MPaa <= pb_MPaa:
        viscosity_cP = saturatedviscosity_cP
    else:
        viscosity_cP = unf_undersaturatedoilviscosity_VB_cP(p_MPaa, pb_MPaa, saturatedviscosity_cP)
    return viscosity_cP



def unf_heat_capacity_oil_Gambill_JkgC(gamma_oil, t_c):
    """
        Oil heat capacity in SI. Gambill correlation

    :param gamma_oil: specific oil density(by water)
    :param t_c: temperature in C
    :return: heat capacity in SI - JkgC

    ref1 Book: Brill J. P., Mukherjee H. K. Multiphase flow in wells. –
    Society of Petroleum Engineers, 1999. – Т. 17. in Page 122
    """

    t_f = uc.C_2_F(t_c)
    heat_capacity_oil_btulbmF = ((0.388 + 0.00045 * t_f) / gamma_oil ** (0.5))
    return uc.btulbmF_2_kJkgK(heat_capacity_oil_btulbmF) * 1000